%
% %%% PRINTER: %%%
% Autor: Wiebke Petersen
% Datum: 26.01.2016
%

:- module(app_printer, [pprint_list/2, pprint_tree/2, pprint_dot/2]).

:- dynamic(node/1).


% pprint_list/2
% pprint_list(+LIST:content, +OBJ:file-stream).
pprint_list([H|T], FS):-
        write(FS, H),
        tab(FS, 1),
        pprint_list(T, FS).

pprint_list([], FS):- nl(FS).


% pprint_tree/2-3
% pprint_tree(+LIST:content, (+INT:tabcount), +OBJ:file-stream).
pprint_tree(Term, FS):-                    
        pprint_tree(Term, 0, FS),
        nl(FS).

pprint_tree(Term, N, FS):-                              % Baum drucken.
        Term =.. [F|Args],                              % Struktur zu Liste.
        tab(FS, N),                                     % // 
        write(FS, F),                                   % // Ausgabe des Mutterknotens.
        nl(FS),                                         % //
        N1 is N + 3,                                    % Tabulator erhoehen.
        pprint_tree_child(Args, N1, FS).                % Unterbaeume drucken.


% pprint_tree_child/2
% pprint_tree_child(+LIST:content, +INT:tabcount, +OBJ:file-stream).
pprint_tree_child([H|T], N, FS):-                       % Unterbaeume drucken.
        pprint_tree(H, N, FS),                          % Drucke eine Schwester.
        pprint_tree_child(T, N, FS).                    % Drucke die anderen Schwestern.

pprint_tree_child([], _, _).                            % Termination.


% pprint_dot/2-3
% pprint_dot(+STR:content, (+INT:id), +OBJ:file-stream).
pprint_dot(Term,FS):-
     retractall(node(_)),
     write(FS, 'digraph G {'),                          % //
     nl(FS),                                            % //
     assert(node(0)),                                   % // Ausgabe des Dokumentes.
     pprint_dot(Term, 0, FS),                           % // digraph G { (...) }
     write(FS, '}').                                    % //

pprint_dot(Term, N, FS):-
     Term =.. [F|Args],                                 % Struktur des Knoten.
     tab(FS, 4),                                        % //
     write(FS, q),                                      % //
     write(FS, N),                                      % //
     write(FS, '[label='),                              % // Ausgabe des Knoten.
     write(FS, F),                                      % // q'_INT'[label='_STR'];
     write(FS, '];'),                                   % //
     nl(FS),                                            % //
     N1 is N+1,                                         % Element ID erhoehen.
     retract(node(_)),
     assert(node(N1)),
     pprint_dot_relation(Args, N, FS).                  % Relation drucken.


% pprint_dot_relation/3
% pprint_dot_relation(+LIST:content, +INT:id, +OBJ:file-stream).
pprint_dot_relation([H|T], Parent, FS):-
     node(Nchild),
     pprint_dot(H, Nchild, FS),                         % //
     tab(FS, 4),                                        % //
     write(FS, q),                                      % //
     write(FS, Parent),                                 % //  Ausgabe der Relation.
     write(FS, '->'),                                   % //  q'_INT'->q'_INT';
     write(FS, q),                                      % //
     write(FS, Nchild),                                 % //
     write(FS, ';'),                                    % //
     nl(FS),
     pprint_dot_relation(T, Parent, FS).

pprint_dot_relation([],_ ,_).                           % Termination.